@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020-2024 Nicotine+ Contributors
+# COPYRIGHT (C) 2020-2025 Nicotine+ Contributors
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
 # COPYRIGHT (C) 2013 eLvErDe <gandalf@le-vert.net>
@@ -37,12 +37,23 @@ except ImportError:
 
 from collections import defaultdict
 
-from pynicotine import slskmessages
 from pynicotine.config import config
 from pynicotine.core import core
 from pynicotine.events import events
 from pynicotine.logfacility import log
+from pynicotine.slskmessages import ConnectionType
+from pynicotine.slskmessages import DownloadFile
+from pynicotine.slskmessages import FileOffset
+from pynicotine.slskmessages import FolderContentsRequest
+from pynicotine.slskmessages import increment_token
+from pynicotine.slskmessages import initial_token
+from pynicotine.slskmessages import PlaceInQueueRequest
+from pynicotine.slskmessages import QueueUpload
+from pynicotine.slskmessages import SetDownloadLimit
+from pynicotine.slskmessages import TransferDirection
 from pynicotine.slskmessages import TransferRejectReason
+from pynicotine.slskmessages import TransferResponse
+from pynicotine.slskmessages import UserStatus
 from pynicotine.transfers import Transfer
 from pynicotine.transfers import Transfers
 from pynicotine.transfers import TransferStatus
@@ -54,7 +65,6 @@ from pynicotine.utils import truncate_string_byte
 
 
 class RequestedFolder:
-
     __slots__ = ("username", "folder_path", "download_folder_path", "request_timer_id", "has_retried",
                  "legacy_attempt")
 
@@ -68,13 +78,16 @@ class RequestedFolder:
 
 
 class Downloads(Transfers):
+    __slots__ = ("_requested_folders", "_requested_folder_token", "_folder_basename_byte_limits",
+                 "_pending_queue_messages", "_download_queue_timer_id", "_retry_connection_downloads_timer_id",
+                 "_retry_io_downloads_timer_id")
 
     def __init__(self):
 
-        super().__init__(transfers_file_path=os.path.join(config.data_folder_path, "downloads.json"))
+        super().__init__(name="downloads")
 
         self._requested_folders = defaultdict(dict)
-        self._requested_folder_token = 0
+        self._requested_folder_token = initial_token()
 
         self._folder_basename_byte_limits = {}
         self._pending_queue_messages = {}
@@ -107,10 +120,11 @@ class Downloads(Transfers):
 
     def _quit(self):
 
+        self._delete_stale_incomplete_downloads()
+
         super()._quit()
 
         self._folder_basename_byte_limits.clear()
-        self._requested_folder_token = 0
 
     def _server_login(self, msg):
 
@@ -119,8 +133,9 @@ class Downloads(Transfers):
 
         super()._server_login(msg)
 
-        # Request queue position of queued downloads every 3 minutes
-        self._download_queue_timer_id = events.schedule(delay=180, callback=self._check_download_queue, repeat=True)
+        # Request queue position of queued downloads every 5 minutes
+        self._download_queue_timer_id = events.schedule(
+            delay=300, callback=self._request_queue_positions, repeat=True)
 
         # Retry downloads failed due to connection issues every 3 minutes
         self._retry_connection_downloads_timer_id = events.schedule(
@@ -253,13 +268,10 @@ class Downloads(Transfers):
         else:
             speed_limit = 0
 
-        core.send_message_to_network_thread(slskmessages.SetDownloadLimit(speed_limit))
+        core.send_message_to_network_thread(SetDownloadLimit(speed_limit))
 
     # Transfer Actions #
 
-    def _append_transfer(self, transfer):
-        self.transfers[transfer.username + transfer.virtual_path] = transfer
-
     def _update_transfer(self, transfer, update_parent=True):
         events.emit("update-download", transfer, update_parent)
 
@@ -284,26 +296,23 @@ class Downloads(Transfers):
             except re.error:
                 pass
 
-        if slskmessages.UserStatus.OFFLINE in (core.users.login_status, core.users.statuses.get(username)):
+        if UserStatus.OFFLINE in (core.users.login_status, core.users.statuses.get(username)):
             # Either we are offline or the user we want to download from is
             self._abort_transfer(transfer, status=TransferStatus.USER_LOGGED_OFF)
             return False
 
-        log.add_transfer("Adding file %(filename)s from user %(user)s to download queue", {
-            "filename": virtual_path,
-            "user": username
-        })
+        log.add_transfer("Adding file %s from user %s to download queue", (virtual_path, username))
 
         _file_path, file_exists = self.get_complete_download_file_path(
             username, virtual_path, size, transfer.folder_path)
 
         if file_exists:
-            self._finish_transfer(transfer, already_exists=True)
+            self._finish_transfer(transfer)
             return False
 
         super()._enqueue_transfer(transfer)
 
-        msg = slskmessages.QueueUpload(virtual_path, transfer.legacy_attempt)
+        msg = QueueUpload(virtual_path, transfer.legacy_attempt)
 
         if not core.shares.initialized:
             # Remain queued locally until our shares have initialized, to prevent invalid
@@ -342,11 +351,14 @@ class Downloads(Transfers):
 
     def _dequeue_transfer(self, transfer):
 
-        super()._dequeue_transfer(transfer)
+        if not super()._dequeue_transfer(transfer):
+            return False
 
         if transfer in self._pending_queue_messages:
             del self._pending_queue_messages[transfer]
 
+        return True
+
     def _file_downloaded_actions(self, username, file_path):
 
         if config.sections["notifications"]["notification_popup_file"]:
@@ -412,27 +424,24 @@ class Downloads(Transfers):
                     "error": error
                 })
 
-    def _move_finished_transfer(self, transfer):
+    def _move_finished_transfer(self, transfer, incomplete_file_path):
 
         download_folder_path = transfer.folder_path or self.get_default_download_folder(transfer.username)
         download_folder_path_encoded = encode_path(download_folder_path)
 
         download_basename = self.get_download_basename(transfer.virtual_path, download_folder_path, avoid_conflict=True)
         download_file_path = os.path.join(download_folder_path, download_basename)
-        incomplete_file_path_encoded = transfer.file_handle.name
-
-        self._close_file(transfer)
 
         try:
             if not os.path.isdir(download_folder_path_encoded):
                 os.makedirs(download_folder_path_encoded)
 
-            shutil.move(incomplete_file_path_encoded, encode_path(download_file_path))
+            shutil.move(incomplete_file_path, encode_path(download_file_path))
 
         except OSError as error:
             log.add(
                 _("Couldn't move '%(tempfile)s' to '%(file)s': %(error)s"), {
-                    "tempfile": incomplete_file_path_encoded.decode("utf-8", "replace"),
+                    "tempfile": incomplete_file_path.decode("utf-8", "replace"),
                     "file": download_file_path,
                     "error": error
                 }
@@ -445,23 +454,22 @@ class Downloads(Transfers):
 
         return download_file_path
 
-    def _finish_transfer(self, transfer, already_exists=False):
+    def _finish_transfer(self, transfer):
 
         username = transfer.username
         virtual_path = transfer.virtual_path
+        already_exists = transfer.file_handle is None
+        incomplete_file_path = transfer.file_handle.name if not already_exists else None
 
-        self._deactivate_transfer(transfer)
+        super()._finish_transfer(transfer)
 
         if not already_exists:
-            download_file_path = self._move_finished_transfer(transfer)
+            download_file_path = self._move_finished_transfer(transfer, incomplete_file_path)
 
             if download_file_path is None:
                 # Download was not moved successfully
                 return
 
-        transfer.status = TransferStatus.FINISHED
-        transfer.current_byte_offset = transfer.size
-
         if not self._auto_clear_transfer(transfer):
             self._update_transfer(transfer)
 
@@ -484,17 +492,9 @@ class Downloads(Transfers):
             }
         )
 
-    def _abort_transfer(self, transfer, denied_message=None, status=None, update_parent=True):
-
-        transfer.legacy_attempt = False
-        transfer.size_changed = False
-
-        if transfer.sock is not None:
-            core.send_message_to_network_thread(slskmessages.CloseConnection(transfer.sock))
+    def _abort_transfer(self, transfer, status=None, denied_message=None, update_parent=True):
 
         if transfer.file_handle is not None:
-            self._close_file(transfer)
-
             log.add_download(
                 _("Download aborted, user %(user)s file %(file)s"), {
                     "user": transfer.username,
@@ -502,55 +502,77 @@ class Downloads(Transfers):
                 }
             )
 
-        self._deactivate_transfer(transfer)
-        self._dequeue_transfer(transfer)
-        self._unfail_transfer(transfer)
+        super()._abort_transfer(transfer, status=status, denied_message=denied_message)
 
-        if not status:
-            return
+        if status:
+            events.emit("abort-download", transfer, status, update_parent)
 
-        transfer.status = status
+    def _clear_transfer(self, transfer, denied_message=None, update_parent=True):
 
-        if status not in {TransferStatus.FINISHED, TransferStatus.FILTERED, TransferStatus.PAUSED}:
-            self._fail_transfer(transfer)
+        virtual_path = transfer.virtual_path
+        username = transfer.username
 
-        events.emit("abort-download", transfer, status, update_parent)
-
-    def _auto_clear_transfer(self, transfer):
-
-        if config.sections["transfers"]["autoclear_downloads"]:
-            self._clear_transfer(transfer)
-            return True
-
-        return False
-
-    def _clear_transfer(self, transfer, update_parent=True):
-
-        self._abort_transfer(transfer)
-
-        log.add_transfer("Clearing download %(path)s from user %(user)s", {
-            "path": transfer.virtual_path,
-            "user": transfer.username
-        })
+        log.add_transfer("Clearing download %s from user %s", (virtual_path, username))
 
         try:
-            del self.transfers[transfer.username + transfer.virtual_path]
+            super()._clear_transfer(transfer, denied_message=denied_message)
 
         except KeyError:
-            log.add(("FIXME: failed to remove download %(path)s from user %(user)s, not "
-                     "present in list"), {
-                "path": transfer.virtual_path,
-                "user": transfer.username
-            })
+            log.add("FIXME: failed to remove download %s from user %s, not present in list",
+                    (virtual_path, username))
 
         events.emit("clear-download", transfer, update_parent)
 
-    def _check_download_queue(self):
+    def _delete_stale_incomplete_downloads(self):
+
+        if not self._allow_saving_transfers:
+            return
+
+        incomplete_download_folder_path = self.get_incomplete_download_folder()
+        allowed_incomplete_file_paths = {
+            encode_path(self.get_incomplete_download_file_path(transfer.username, transfer.virtual_path))
+            for transfer in self.transfers.values()
+            if transfer.current_byte_offset and transfer.status != TransferStatus.FINISHED
+        }
+        prefix = b"INCOMPLETE"
+        prefix_len = len(prefix)
+        md5_len = 32
+        md5_regex = re.compile(b"[0-9a-f]{32}", re.IGNORECASE)
+
+        try:
+            with os.scandir(encode_path(incomplete_download_folder_path)) as entries:
+                for entry in entries:
+                    if entry.is_dir():
+                        continue
+
+                    if entry.path in allowed_incomplete_file_paths:
+                        continue
+
+                    basename = entry.name
+
+                    # Skip files that are not incomplete downloads
+                    if (not basename.startswith(prefix)
+                            or len(basename) <= (prefix_len + md5_len)
+                            or not md5_regex.match(basename[prefix_len:prefix_len + md5_len])):
+                        continue
+
+                    # Incomplete file no longer has a download associated with it. Delete it.
+                    try:
+                        os.remove(entry.path)
+                        log.add_transfer("Deleted stale incomplete download %s", entry.path)
+
+                    except OSError as error:
+                        log.add_transfer("Cannot delete incomplete download %s: %s", (entry.path, error))
+
+        except OSError as error:
+            log.add_transfer("Cannot read incomplete download folder: %s", error)
+
+    def _request_queue_positions(self):
 
         for download in self.queued_transfers:
             core.send_message_to_peer(
                 download.username,
-                slskmessages.PlaceInQueueRequest(download.virtual_path, download.legacy_attempt)
+                PlaceInQueueRequest(download.virtual_path, download.legacy_attempt)
             )
 
     def _retry_failed_connection_downloads(self):
@@ -612,7 +634,7 @@ class Downloads(Transfers):
         # Remove parent folders of the requested folder from path
         parent_folder_path = root_folder_path if root_folder_path else folder_path
         removed_parent_folders = parent_folder_path.rpartition("\\")[0]
-        target_folders = folder_path.replace(removed_parent_folders, "").lstrip("\\").replace("\\", os.sep)
+        target_folders = folder_path.replace(removed_parent_folders, "", 1).lstrip("\\").replace("\\", os.sep)
 
         # Check if a custom download location was specified
         if not download_folder_path:
@@ -662,7 +684,7 @@ class Downloads(Transfers):
         basename = clean_file(virtual_path.rpartition("\\")[-1])
         basename_no_extension, separator, extension = basename.rpartition(".")
         extension = separator + extension
-        basename_limit = max_bytes - len(extension.encode("utf-8"))
+        basename_limit = max_bytes - len(extension.encode())
         basename_no_extension = truncate_string_byte(basename_no_extension, max(0, basename_limit))
 
         if basename_limit < 0:
@@ -711,7 +733,7 @@ class Downloads(Transfers):
         transferring."""
 
         md5sum = md5()
-        md5sum.update((virtual_path + username).encode("utf-8"))
+        md5sum.update((virtual_path + username).encode())
         prefix = f"INCOMPLETE{md5sum.hexdigest()}"
 
         # Ensure file name length doesn't exceed file system limit
@@ -721,7 +743,7 @@ class Downloads(Transfers):
         basename = clean_file(virtual_path.rpartition("\\")[-1])
         basename_no_extension, separator, extension = basename.rpartition(".")
         extension = separator + extension
-        basename_limit = max_bytes - len(prefix) - len(extension.encode("utf-8"))
+        basename_limit = max_bytes - len(prefix) - len(extension.encode())
         basename_no_extension = truncate_string_byte(basename_no_extension, max(0, basename_limit))
 
         if basename_limit < 0:
@@ -761,15 +783,12 @@ class Downloads(Transfers):
             delay=timeout, callback=self._requested_folder_timeout, callback_args=(requested_folder,)
         )
 
-        log.add_transfer("Requesting contents of folder %(path)s from user %(user)s", {
-            "path": folder_path,
-            "user": username
-        })
+        log.add_transfer("Requesting contents of folder %s from user %s", (folder_path, username))
 
-        self._requested_folder_token = slskmessages.increment_token(self._requested_folder_token)
+        self._requested_folder_token = increment_token(self._requested_folder_token)
 
         core.send_message_to_peer(
-            username, slskmessages.FolderContentsRequest(
+            username, FolderContentsRequest(
                 folder_path, self._requested_folder_token, legacy_client=requested_folder.legacy_attempt
             )
         )
@@ -884,7 +903,7 @@ class Downloads(Transfers):
             # Skip redundant status updates from users in joined rooms
             return
 
-        if msg.status == slskmessages.UserStatus.OFFLINE:
+        if msg.status == UserStatus.OFFLINE:
             for users in (self.queued_users, self.failed_users):
                 for download in users.get(username, {}).copy().values():
                     self._abort_transfer(download, status=TransferStatus.USER_LOGGED_OFF)
@@ -897,37 +916,40 @@ class Downloads(Transfers):
             return
 
         # No need to check transfers on away status change
-        if username not in self._online_users:
-            for download in self.failed_users.get(username, {}).copy().values():
-                if download.status != TransferStatus.USER_LOGGED_OFF:
-                    # Only a online/away status update, no transfers to resume
-                    break
+        if username in self._online_users:
+            return
 
-                # User logged in, resume "User logged off" transfers
-                self._unfail_transfer(download)
+        # User logged in, resume "User logged off" transfers
+        for download in self.failed_users.get(username, {}).copy().values():
+            self._unfail_transfer(download)
 
-                if self._enqueue_transfer(download):
-                    self._update_transfer(download)
+            if self._enqueue_transfer(download):
+                self._update_transfer(download)
 
-            self._online_users.add(username)
+        self._online_users.add(username)
 
     def _set_connection_stats(self, download_bandwidth=0, **_unused):
         self.total_bandwidth = download_bandwidth
 
-    def _peer_connection_error(self, username, msgs=None, is_offline=False, is_timeout=True):
+    def _peer_connection_error(self, username, conn_type, msgs, is_offline=False, is_timeout=True):
 
-        if msgs is None:
+        if not msgs:
             return
 
+        if conn_type not in {ConnectionType.FILE, ConnectionType.PEER}:
+            return
+
+        failed_msg_types = {QueueUpload, PlaceInQueueRequest}
+
         for msg in msgs:
-            if msg.__class__ is slskmessages.QueueUpload:
+            if msg.__class__ in failed_msg_types:
                 self._cant_connect_queue_file(username, msg.file, is_offline, is_timeout)
 
-    def _peer_connection_closed(self, username, msgs=None):
-        self._peer_connection_error(username, msgs, is_timeout=False)
+    def _peer_connection_closed(self, username, conn_type, msgs=None):
+        self._peer_connection_error(username, conn_type, msgs, is_timeout=False)
 
     def _cant_connect_queue_file(self, username, virtual_path, is_offline, is_timeout):
-        """We can't connect to the user, either way (QueueUpload)."""
+        """We can't connect to the user, either way (QueueUpload, PlaceInQueueRequest)."""
 
         download = self.queued_users.get(username, {}).get(virtual_path)
 
@@ -943,12 +965,8 @@ class Downloads(Transfers):
         else:
             status = TransferStatus.CONNECTION_CLOSED
 
-        log.add_transfer(("Download attempt for file %(filename)s from user %(user)s failed "
-                          "with status %(status)s"), {
-            "filename": virtual_path,
-            "user": username,
-            "status": status
-        })
+        log.add_transfer("Download attempt for file %s from user %s failed with status %s",
+                         (virtual_path, username, status))
         self._abort_transfer(download, status=status)
 
     def _requested_folder_timeout(self, requested_folder):
@@ -961,19 +979,13 @@ class Downloads(Transfers):
         folder_path = requested_folder.folder_path
 
         if requested_folder.has_retried:
-            log.add_transfer(("Folder content request for folder %(path)s from user %(user)s timed out, "
-                              "giving up"), {
-                "path": folder_path,
-                "user": username
-            })
+            log.add_transfer("Folder content request for folder %s from user %s timed out, "
+                             "giving up", (folder_path, username))
             del self._requested_folders[username][folder_path]
             return
 
-        log.add_transfer(("Folder content request for folder %(path)s from user %(user)s timed out, "
-                          "retrying"), {
-            "path": folder_path,
-            "user": username
-        })
+        log.add_transfer("Folder content request for folder %s from user %s timed out, "
+                         "retrying", (folder_path, username))
 
         requested_folder.has_retried = True
         self.enqueue_folder(username, folder_path, requested_folder.download_folder_path)
@@ -982,26 +994,34 @@ class Downloads(Transfers):
         """Peer code 37."""
 
         username = msg.username
+        folder_path = msg.dir
 
         if username not in self._requested_folders:
             return
 
-        for folder_path, files in msg.list.items():
-            if folder_path not in self._requested_folders[username]:
+        requested_folder = self._requested_folders[username].get(msg.dir)
+
+        if requested_folder is None:
+            return
+
+        log.add_transfer("Received response for folder content request for folder %s "
+                         "from user %s", (folder_path, username))
+
+        if requested_folder.request_timer_id is not None:
+            events.cancel_scheduled(requested_folder.request_timer_id)
+            requested_folder.request_timer_id = None
+
+        if not msg.list and not requested_folder.legacy_attempt:
+            log.add_transfer("Folder content response is empty. Trying legacy latin-1 request.")
+            requested_folder.legacy_attempt = True
+            self.enqueue_folder(username, folder_path, requested_folder.download_folder_path)
+            return
+
+        for i_folder_path, files in msg.list.items():
+            if i_folder_path != folder_path:
                 continue
 
-            log.add_transfer(("Received response for folder content request for folder %(path)s "
-                              "from user %(user)s"), {
-                "path": folder_path,
-                "user": username
-            })
-
             num_files = len(files)
-            requested_folder = self._requested_folders[username][folder_path]
-
-            if requested_folder.request_timer_id is not None:
-                events.cancel_scheduled(requested_folder.request_timer_id)
-                requested_folder.request_timer_id = None
 
             if check_num_files and num_files > 100:
                 check_num_files = False
@@ -1011,21 +1031,10 @@ class Downloads(Transfers):
                 )
                 return
 
-            if not files and not requested_folder.legacy_attempt:
-                log.add_transfer("Folder content response is empty. Trying legacy latin-1 request.")
-                requested_folder.legacy_attempt = requested_folder.has_retried = True
-                self.enqueue_folder(username, folder_path, requested_folder.download_folder_path)
-                return
-
             destination_folder_path = self.get_folder_destination(username, folder_path)
-            del self._requested_folders[username][folder_path]
 
-            log.add_transfer(("Attempting to download files in folder %(folder)s for user %(user)s. "
-                              "Destination path: %(destination)s"), {
-                "folder": folder_path,
-                "user": username,
-                "destination": destination_folder_path
-            })
+            log.add_transfer("Attempting to download files in folder %s for user %s. "
+                             "Destination path: %s", (folder_path, username, destination_folder_path))
 
             for _code, basename, file_size, _ext, file_attributes, *_unused in files:
                 virtual_path = folder_path.rstrip("\\") + "\\" + basename
@@ -1034,20 +1043,20 @@ class Downloads(Transfers):
                     username, virtual_path, folder_path=destination_folder_path, size=file_size,
                     file_attributes=file_attributes)
 
+        del self._requested_folders[username][folder_path]
+
     def _transfer_request(self, msg):
         """Peer code 40."""
 
-        if msg.direction != slskmessages.TransferDirection.UPLOAD:
+        if msg.direction != TransferDirection.UPLOAD:
             return
 
         username = msg.username
         response = self._transfer_request_downloads(msg)
 
-        log.add_transfer(("Responding to download request with token %(token)s for file %(filename)s "
-                          "from user: %(user)s, allowed: %(allowed)s, reason: %(reason)s"), {
-            "token": response.token, "filename": msg.file, "user": username,
-            "allowed": response.allowed, "reason": response.reason
-        })
+        log.add_transfer("Responding to download request with token %s for file %s "
+                         "from user: %s, allowed: %s, reason: %s",
+                         (response.token, msg.file, username, response.allowed, response.reason))
 
         core.send_message_to_peer(username, response)
 
@@ -1058,11 +1067,8 @@ class Downloads(Transfers):
         size = msg.filesize
         token = msg.token
 
-        log.add_transfer("Received download request with token %(token)s for file %(filename)s from user %(user)s", {
-            "token": token,
-            "filename": virtual_path,
-            "user": username
-        })
+        log.add_transfer("Received download request with token %s for file %s from user %s",
+                         (token, virtual_path, username))
 
         download = (self.queued_users.get(username, {}).get(virtual_path)
                     or self.failed_users.get(username, {}).get(virtual_path))
@@ -1088,7 +1094,7 @@ class Downloads(Transfers):
             self._activate_transfer(download, token)
             self._update_transfer(download)
 
-            return slskmessages.TransferResponse(allowed=True, token=token)
+            return TransferResponse(allowed=True, token=token)
 
         download = self.transfers.get(username + virtual_path)
         cancel_reason = TransferRejectReason.CANCELLED
@@ -1117,27 +1123,20 @@ class Downloads(Transfers):
                 self._activate_transfer(transfer, token)
                 self._update_transfer(transfer)
 
-                return slskmessages.TransferResponse(allowed=True, token=token)
+                return TransferResponse(allowed=True, token=token)
 
-        log.add_transfer("Denied file request: User %(user)s, %(msg)s", {
-            "user": username,
-            "msg": msg
-        })
-
-        return slskmessages.TransferResponse(allowed=False, reason=cancel_reason, token=token)
+        log.add_transfer("Denied file request: user %s, message %s", (username, msg))
+        return TransferResponse(allowed=False, reason=cancel_reason, token=token)
 
     def _transfer_timeout(self, transfer):
 
         if transfer.request_timer_id is None:
             return
 
-        log.add_transfer("Download %(filename)s with token %(token)s for user %(user)s timed out", {
-            "filename": transfer.virtual_path,
-            "token": transfer.token,
-            "user": transfer.username
-        })
+        log.add_transfer("Download %s with token %s for user %s timed out",
+                         (transfer.virtual_path, transfer.token, transfer.username))
 
-        self._abort_transfer(transfer, status=TransferStatus.CONNECTION_TIMEOUT)
+        super()._transfer_timeout(transfer)
 
     def _download_file_error(self, username, token, error):
         """Networking thread encountered a local file error for download."""
@@ -1170,12 +1169,8 @@ class Downloads(Transfers):
         need_update = True
         download_started = False
 
-        log.add_transfer(("Received file download init with token %(token)s for file %(filename)s "
-                          "from user %(user)s"), {
-            "token": token,
-            "filename": virtual_path,
-            "user": username
-        })
+        log.add_transfer("Received file download init with token %s for file %s from user %s",
+                         (token, virtual_path, username))
 
         try:
             incomplete_folder_path_encoded = encode_path(incomplete_folder_path)
@@ -1216,8 +1211,8 @@ class Downloads(Transfers):
         else:
             download.file_handle = file_handle
             download.last_byte_offset = offset
-            download.last_update = time.monotonic()
-            download.start_time = download.last_update - download.time_elapsed
+            download.start_time = time.monotonic() - download.time_elapsed
+            download.retry_attempt = False
 
             core.statistics.append_stat_value("started_downloads", 1)
             download_started = True
@@ -1231,10 +1226,10 @@ class Downloads(Transfers):
 
             if download.size > offset:
                 download.status = TransferStatus.TRANSFERRING
-                core.send_message_to_network_thread(slskmessages.DownloadFile(
+                core.send_message_to_network_thread(DownloadFile(
                     sock=sock, token=token, file=file_handle, leftbytes=(download.size - offset)
                 ))
-                core.send_message_to_peer(username, slskmessages.FileOffset(sock, offset))
+                core.send_message_to_peer(username, FileOffset(sock, offset))
 
             else:
                 self._finish_transfer(download)
@@ -1267,12 +1262,8 @@ class Downloads(Transfers):
             # The peer is possibly using an old client that doesn't support Unicode
             # (Soulseek NS). Attempt to request file name encoded as latin-1 once.
 
-            log.add_transfer("User %(user)s responded with reason '%(reason)s' for download request %(filename)s. "
-                             "Attempting to request file as latin-1.", {
-                                 "user": username,
-                                 "reason": reason,
-                                 "filename": virtual_path
-                             })
+            log.add_transfer("User %s responded with reason '%s' for download request %s. "
+                             "Attempting to request file as latin-1.", (username, reason, virtual_path))
 
             self._dequeue_transfer(download)
             download.legacy_attempt = True
@@ -1291,11 +1282,8 @@ class Downloads(Transfers):
         self._abort_transfer(download, status=reason)
         self._update_transfer(download)
 
-        log.add_transfer("Download request denied by user %(user)s for file %(filename)s. Reason: %(reason)s", {
-            "user": username,
-            "filename": virtual_path,
-            "reason": msg.reason
-        })
+        log.add_transfer("Download request denied by user %s for file %s. Reason: %s",
+                         (username, virtual_path, msg.reason))
 
     def _upload_failed(self, msg):
         """Peer code 46."""
@@ -1307,16 +1295,24 @@ class Downloads(Transfers):
         if download is None:
             return
 
-        if download.token not in self.active_users.get(username, {}):
+        if (download.token not in self.active_users.get(username, {})
+                and virtual_path not in self.failed_users.get(username, {})
+                and virtual_path not in self.queued_users.get(username, {})):
             return
 
-        should_retry = not download.legacy_attempt
+        if download.status in {TransferStatus.DOWNLOAD_FOLDER_ERROR, TransferStatus.LOCAL_FILE_ERROR}:
+            # Local error, no need to retry
+            return
 
-        if should_retry:
+        if not download.retry_attempt:
             # Attempt to request file name encoded as latin-1 once
 
-            self._dequeue_transfer(download)
-            download.legacy_attempt = True
+            # We mark download as failed when aborting it, to avoid a redundant request
+            # to unwatch the user. Need to call _unfail_transfer() to undo this.
+            self._abort_transfer(download, status=TransferStatus.CONNECTION_CLOSED)
+            self._unfail_transfer(download)
+
+            download.legacy_attempt = download.retry_attempt = True
 
             if self._enqueue_transfer(download):
                 self._update_transfer(download)
@@ -1325,14 +1321,12 @@ class Downloads(Transfers):
 
         # Already failed once previously, give up
         self._abort_transfer(download, status=TransferStatus.CONNECTION_CLOSED)
+        download.retry_attempt = False
 
-        log.add_transfer("Upload attempt by user %(user)s for file %(filename)s failed. Reason: %(reason)s", {
-            "filename": virtual_path,
-            "user": username,
-            "reason": download.status
-        })
+        log.add_transfer("Upload attempt by user %s for file %s failed. Reason: %s",
+                         (virtual_path, username, download.status))
 
-    def _file_download_progress(self, username, token, bytes_left):
+    def _file_download_progress(self, username, token, bytes_left, speed=None):
         """A file download is in progress."""
 
         download = self.active_users.get(username, {}).get(token)
@@ -1344,26 +1338,10 @@ class Downloads(Transfers):
             events.cancel_scheduled(download.request_timer_id)
             download.request_timer_id = None
 
-        current_time = time.monotonic()
-        size = download.size
-
-        download.status = TransferStatus.TRANSFERRING
-        download.time_elapsed = current_time - download.start_time
-        download.current_byte_offset = current_byte_offset = (size - bytes_left)
-        byte_difference = current_byte_offset - download.last_byte_offset
-
-        if byte_difference:
-            core.statistics.append_stat_value("downloaded_size", byte_difference)
-
-            if size > current_byte_offset or download.speed <= 0:
-                download.speed = int(max(0, byte_difference // max(0.1, current_time - download.last_update)))
-                download.time_left = (size - current_byte_offset) // download.speed if download.speed else 0
-            else:
-                download.time_left = 0
-
-        download.last_byte_offset = current_byte_offset
-        download.last_update = current_time
-
+        self._update_transfer_progress(
+            download, stat_id="downloaded_size",
+            current_byte_offset=(download.size - bytes_left), speed=speed
+        )
         self._update_transfer(download)
 
     def _file_connection_closed(self, username, token, sock, **_unused):
@@ -1381,7 +1359,7 @@ class Downloads(Transfers):
             self._finish_transfer(download)
             return
 
-        if core.users.statuses.get(download.username) == slskmessages.UserStatus.OFFLINE:
+        if core.users.statuses.get(download.username) == UserStatus.OFFLINE:
             status = TransferStatus.USER_LOGGED_OFF
         else:
             status = TransferStatus.CANCELLED
