@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2021-2022 Nicotine+ Contributors
+# COPYRIGHT (C) 2021-2023 Nicotine+ Contributors
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -16,31 +16,22 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk
-
+from pynicotine.gtkgui.widgets import ui
 from pynicotine.gtkgui.widgets.dialogs import Dialog
-from pynicotine.gtkgui.widgets.ui import UserInterface
 
 
-class Shortcuts(UserInterface, Dialog):
+class Shortcuts(Dialog):
 
-    def __init__(self, frame):
+    def __init__(self, application):
 
-        UserInterface.__init__(self, "ui/dialogs/shortcuts.ui")
-        self.dialog, self.emoji_shortcut = self.widgets
+        self.dialog, self.emoji_shortcut = ui.load(scope=self, path="dialogs/shortcuts.ui")
 
-        Dialog.__init__(
-            self,
-            dialog=self.dialog,
-            parent=frame.window,
-            close_destroy=False
+        super().__init__(
+            widget=self.dialog,
+            parent=application.window
         )
-        frame.window.set_help_overlay(self.dialog)
-
-        if hasattr(Gtk.Entry.props, "show-emoji-icon"):
-            # Emoji picker only available in GTK 3.24+
-            self.emoji_shortcut.show()
+        application.window.set_help_overlay(self.dialog)
 
         # Workaround for off-centered dialog on first run
-        self.dialog.show()
-        self.dialog.hide()
+        self.dialog.set_visible(True)
+        self.dialog.set_visible(False)
