@@ -16,7 +16,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from pynicotine.config import config
 from pynicotine.core import core
 from pynicotine.events import events
 from pynicotine.gtkgui.widgets import ui
@@ -63,9 +62,13 @@ class WishList(Dialog):
             }
         )
 
-        for wish in config.sections["server"]["autosearch"]:
-            wish = str(wish)
-            self.list_view.add_row([wish], select_row=False)
+        self.list_view.freeze()
+
+        for search_item in core.search.searches.values():
+            if search_item.mode == "wishlist":
+                self.add_wish(search_item.term, select=False)
+
+        self.list_view.unfreeze()
 
         self.completion_entry = CompletionEntry(self.wish_entry, self.list_view.model)
         Accelerator("<Shift>Tab", self.list_view.widget, self.on_list_focus_entry_accelerator)  # skip column header
@@ -145,15 +148,11 @@ class WishList(Dialog):
         for iterator in self.list_view.get_selected_rows():
             wish = self.list_view.get_row_value(iterator, "wish")
             core.search.do_search(wish, mode="global")
-
-            # Close dialog to discourage manually searching many items in a row
-            # (can result in a temporary ban from the server in extreme cases)
-            self.close()
             return
 
     def on_remove_wish(self, *_args):
 
-        for iterator in reversed(self.list_view.get_selected_rows()):
+        for iterator in reversed(list(self.list_view.get_selected_rows())):
             wish = self.list_view.get_row_value(iterator, "wish")
             core.search.remove_wish(wish)
 
@@ -177,8 +176,8 @@ class WishList(Dialog):
             callback=self.clear_wishlist_response
         ).present()
 
-    def add_wish(self, wish):
-        self.list_view.add_row([wish])
+    def add_wish(self, wish, select=True):
+        self.list_view.add_row([wish], select_row=select)
 
     def remove_wish(self, wish):
 
